﻿using System;
using System.Threading;
using DasConzoleCore.UI;

namespace LyokoCMD.Sim.Workers
{
    /// <summary>
    /// Décrit un programme en arrière-plan du super-calculateur.
    /// </summary>
    public abstract class SCWorker : ITableLineOutput
    {
        protected readonly string _Name;
        public string Name => _Name;
        protected internal SCWorkerStatus Status;

        public bool Running => Status == SCWorkerStatus.Running ||
                               Status == SCWorkerStatus.Suspended ||
                               Status == SCWorkerStatus.Waiting;

        private readonly Thread _thread;

        public SCWorker(string name)
        {
            _Name = name;
            Status = SCWorkerStatus.Ready;

            _thread = new Thread(() =>
            {
                while (Status != SCWorkerStatus.Stopped)
                {
                    if (Status == SCWorkerStatus.Running)
                    {
                        Run();
                    } else if (Status == SCWorkerStatus.Waiting && CanDeWait())
                    {
                        Status = SCWorkerStatus.Running;
                    }

                    Thread.Sleep(100);
                }
            });
        }

        protected virtual bool CanDeWait()
        {
            return false;
        }

        public abstract double RequiredEnergy { get; }

        public void Start()
        {
            if (!Running && Status != SCWorkerStatus.Ready)
            {
                throw new InvalidOperationException("Ce Worker n'est pas prêt pour démarrer");
            }

            PreStart();
            if (!Running)
            {
                _thread.Start();
            }
            this.Status = SCWorkerStatus.Running;
            PostStart();
        }
        protected virtual void PreStart() {}
        protected virtual void PostStart() {}

        public void Suspend()
        {
            if (!Running)
            {
                throw new InvalidOperationException("Ce Worker n'est pas démarré");
            }

            PreSuspend();
            this.Status = SCWorkerStatus.Suspended;
            PostSuspend();
        }
        protected virtual void PreSuspend() {}
        protected virtual void PostSuspend() {}

        public void Stop()
        {
            if (!Running)
            {
                throw new InvalidOperationException("Ce Worker n'est pas démarré");
            }

            PreStop();
            this.Status = SCWorkerStatus.Stopped;
            PostStop();
        }
        protected virtual void PreStop() {}
        protected virtual void PostStop() {}

        /// <summary>
        /// Stop a worker by force. Only intended for internal use.
        /// </summary>
        public void ForceStop()
        {
            this.Status = SCWorkerStatus.Stopped;
        }

        protected abstract void Run();

        public string[] GetColumnNames() => new string[] { "Nom", "Énergie", "Statut" };
        public string[] GetLine() => new string[] { Name, RequiredEnergy.ToString(), Status.ToString() };
    }
}
